<?php

namespace Intergo\Helpers;

use Intergo\Helpers\Calculators\SMSCounter;

class PhoneHelper
{
    /**
     * @param $phoneNumber
     * @param bool $sanitize
     * @return string
     */
    public static function cleanUpPhoneNumber($phoneNumber, bool $sanitize = false)
    {
        $hasPlusSign = self::startsWithPlusSign($phoneNumber);
        $phoneNumber = self::removeWhitespacesAndNonDigitCharacters($phoneNumber);
        if ($sanitize) {
            $phoneNumber = self::sanitizeToGSM($phoneNumber);
        }

        if (!$hasPlusSign) {
            if ($result = self::checkAndReplaceLeadingZerosWithPlus($phoneNumber)) {
                return $result;
            }
            // If it doesn't start with + sign and neither 00 then prepend + sign
            return '+' . $phoneNumber;
        }

        if ($result = self::checkAndRemoveZerosAfterPlus($phoneNumber)) {
            return $result;
        }
        return $phoneNumber;
    }

    /**
     * Prepend Plus sign on number if plus sign not exists already
     *
     * @param string $phoneNumber
     *
     * @return string
     */
    public static function prependPlusSign(string $phoneNumber)
    {
        return self::startsWithPlusSign($phoneNumber) ? $phoneNumber : '+' . $phoneNumber;
    }

    /**
     * This is to replace Zero on beginning of the phone number with the plus sign
     *
     * @param string $phone
     *
     * @return string|string[]|null
     */
    public static function replaceZeroWithPlusSign(string $phone)
    {
        $phone = preg_replace("/\s+/", "", $phone);
        if (preg_match("~^00\d+$~", $phone)) {
            $phone = preg_replace('/^00?/', "+", $phone);
        } elseif (preg_match("~\+00\d+$~", $phone)) {
            $phone = preg_replace('/^\+00?/', "+", $phone);
        }
        return $phone;
    }

    /**
     * @param $number
     * @return bool
     */
    public static function startsWithPlusSign($number)
    {
        return substr($number, 0, 1) == '+';
    }

    /**
     * @param $number
     * @return string
     */
    public static function removeWhitespacesAndNonDigitCharacters($number)
    {
        return preg_replace("/\s+[^0-9]/", "", $number);
    }

    /**
     * If it starts with 00 then replace with + sign
     *
     * @param $number
     * @return string|false
     */
    public static function checkAndReplaceLeadingZerosWithPlus($number)
    {
        if (preg_match("~^00\d+$~", $number)) {
            return preg_replace('/^00?/', "+", $number);
        }
        return false;
    }

    /**
     * If it starts with +00 (following any number of 0s) remove 0s and leave only + sign
     *
     * @param $number
     * @return string|false
     */
    public static function checkAndRemoveZerosAfterPlus($number)
    {
        if (preg_match("~\+00\d+$~", $number)) {
            return preg_replace('/^\+00?/', "+", $number);
        }
        return false;
    }

    /**
     * @param $number
     * @return array|string|string[]
     */
    private static function sanitizeToGSM($number)
    {
        $smsCounter = new SMSCounter();
        $number = $smsCounter->sanitizeToGSM($number);
        $number = preg_replace('/[\x00-\x08\x0B\x0C\x0E-\x1F\x7F-\x9F]/u', '', $number);
        $number = preg_replace('/[\x00-\x08\x0B\x0C\x0E-\x1F\x7F]/u', '', $number);
        $number = preg_replace('/[\x00-\x1F\x7F-\xA0\xAD]/u', '', $number);
        $number = preg_replace('/[^[:print:]]/', '', $number);
        return str_replace([' ', '-', '(', ')'], '', $number);
    }
}
